@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/main.c,v 1.61 2010/02/13 11:23:55 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/main.c,v 1.64 2010/04/13 04:14:08 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
@@ -399,7 +399,16 @@ int main(int argc, char **argv)
 		inet_fd = create_inet_domain_socket(pool_config->listen_addresses, pool_config->port);
 	}
 
-	size = pool_config->num_init_children * pool_config->max_pool * sizeof(ConnectionInfo);
+	/*
+	 * con_info is a 3 dimension array: i corresponds to pgpool child
+	 * process, j corresponds to connection pool in each process and k
+	 * corresponds to backends in each connection pool.
+	 *
+	 * XXX: Before 2010/4/12 this was a 2 dimension array: i
+	 * corresponds to pgpool child process, j corresponds to
+	 * connection pool in each process. Of course this was wrong.
+	 */
+	size = pool_config->num_init_children * pool_config->max_pool * MAX_NUM_BACKENDS * sizeof(ConnectionInfo);
 	con_info = pool_shared_memory_create(size);
 	if (con_info == NULL)
 	{
@@ -418,7 +427,7 @@ int main(int argc, char **argv)
 	memset(pids, 0, size);
 	for (i = 0; i < pool_config->num_init_children; i++)
 	{
-		pids[i].connection_info = &con_info[i * pool_config->max_pool];
+		pids[i].connection_info = &con_info[i * pool_config->max_pool * MAX_NUM_BACKENDS];
 	}
 
 	/* create fail over/switch over event area */
@@ -1509,7 +1518,7 @@ int health_check(void)
 	char kind;
 	int i;
 
-	/* Do health check during recovery */
+	/* Do not execute health check during recovery */
 	if (*InRecovery)
 		return 0;
 
@@ -1538,9 +1547,9 @@ int health_check(void)
 			continue;
 
 		if (*(BACKEND_INFO(i).backend_hostname) == '\0')
-			fd = connect_unix_domain_socket(i);
+			fd = connect_unix_domain_socket(i, FALSE);
 		else
-			fd = connect_inet_domain_socket(i);
+			fd = connect_inet_domain_socket(i, FALSE);
 
 		if (fd < 0)
 		{
@@ -1650,9 +1659,9 @@ system_db_health_check(void)
 		return 0;
 
 	if (*SYSDB_INFO->hostname == '\0')
-		fd = connect_unix_domain_socket_by_port(SYSDB_INFO->port, pool_config->backend_socket_dir);
+		fd = connect_unix_domain_socket_by_port(SYSDB_INFO->port, pool_config->backend_socket_dir, FALSE);
 	else
-		fd = connect_inet_domain_socket_by_port(SYSDB_INFO->hostname, SYSDB_INFO->port);
+		fd = connect_inet_domain_socket_by_port(SYSDB_INFO->hostname, SYSDB_INFO->port, FALSE);
 
 	if (fd < 0)
 	{
