@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/pool_timestamp.c,v 1.3 2009/11/13 23:35:38 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool_timestamp.c,v 1.7 2010/02/17 14:00:14 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
@@ -265,7 +265,7 @@ get_current_timestamp(POOL_CONNECTION_POOL *backend)
 	POOL_STATUS		 status;
 	static char		timestamp[32];
 
-	status = do_query(MASTER(backend), "SELECT now()", &res);
+	status = do_query(MASTER(backend), "SELECT now()", &res, MAJOR(backend));
 	if (status != POOL_CONTINUE)
 	{
 		pool_error("get_current_timestamp: do_query faild");
@@ -636,7 +636,7 @@ rewrite_timestamp(POOL_CONNECTION_POOL *backend, Node *node,
 			int				 i;
 			PrepareStmt		*p_stmt = (PrepareStmt *) node;
 
-			for (i = ctx.num_params; i <= portal->num_tsparams; i++)
+			for (i = 0; i < portal->num_tsparams; i++)
 				p_stmt->argtypes =
 				   	lappend(p_stmt->argtypes, SystemTypeName("timestamptz"));
 		}
@@ -666,7 +666,8 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend, Portal *portal,
 		const char *orig_msg, int *len)
 {
 	int16		 tmp2,
-				 num_params;
+				 num_params,
+				 num_formats;
 	int32		 tmp4;
 	int			 i,
 				 ts_len,
@@ -676,6 +677,14 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend, Portal *portal,
 				*copy_to,
 				*new_msg;
 
+#ifdef TIMESTAMPDEBUG
+	fprintf(stderr, "message length:%d\n", *len);
+	for(i=0;i<*len;i++)
+	{
+		fprintf(stderr, "%02x ", orig_msg[i]);
+	}
+#endif
+
 	ts = get_current_timestamp(backend);
 	if (ts == NULL)
 	{
@@ -686,7 +695,7 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend, Portal *portal,
 	ts_len = strlen(ts);
 
 	*len += (strlen(ts) + sizeof(int32)) * portal->num_tsparams;
-	new_msg = copy_to = (char *) malloc(*len);
+	new_msg = copy_to = (char *) malloc(*len + portal->num_tsparams * sizeof(int16));
 	copy_from = orig_msg;
 
 	/* portal_name */
@@ -702,11 +711,30 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend, Portal *portal,
 	/* format code */
 	memcpy(&tmp2, copy_from, sizeof(int16));
 	copy_len = sizeof(int16);
-	tmp2 = ntohs(tmp2);
-	copy_len += tmp2 * sizeof(int16);
-	memcpy(copy_to, copy_from, copy_len);
+	tmp2 = num_formats = ntohs(tmp2);
+
+	if (num_formats > 1)
+	{
+		/* enlarge message length */
+		*len += portal->num_tsparams * sizeof(int16);
+		tmp2 += portal->num_tsparams;
+	}
+	tmp2 = htons(tmp2);
+	memcpy(copy_to, &tmp2, copy_len);	/* copy number of format codes */
 	copy_to += copy_len; copy_from += copy_len;
 
+	copy_len = num_formats * sizeof(int16);
+
+	memcpy(copy_to, copy_from, copy_len);		/* copy format codes */
+	copy_to += copy_len; copy_from += copy_len;
+
+	if (num_formats > 1)
+	{
+		/* set format codes to zero(text) */
+		memset(copy_to, 0, portal->num_tsparams * 2);
+		copy_to += sizeof(int16) * portal->num_tsparams;
+	}
+
 	/* num params */
 	memcpy(&tmp2, copy_from, sizeof(int16));
 	copy_len = sizeof(int16);
@@ -720,8 +748,16 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend, Portal *portal,
 	for (i = 0; i < num_params; i++)
 	{
 		memcpy(&tmp4, copy_from + copy_len, sizeof(int32));
+		tmp4 = ntohl(tmp4);		/* param length */
 		copy_len += sizeof(int32);
-		copy_len += ntohl(tmp4);
+
+		/* If param length is -1, it indicates that the value is NULL
+		 * and we don't have value slot. So we don't add up copy_len.
+		 */
+		if (tmp4 > 0)
+		{
+			copy_len += tmp4;
+		}
 	}
 	memcpy(copy_to, copy_from, copy_len);
 	copy_to += copy_len; copy_from += copy_len;
